#!/usr/bin/env python

"""
Common classes.

Copyright (C) 2007, 2008, 2009, 2010 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

try:
    set
except NameError:
    from sets import Set as set

# Errors.

class ProcessingError(Exception):

    "A processing error."

    pass

class TableError(ProcessingError):

    "An error occurring during access to a lookup table."

    pass

class TableGenerationError(ProcessingError):

    "An error occurring when generating a lookup table."

    pass

class NodeProcessingError(ProcessingError):

    "A processing error associated with a particular program node."

    def __init__(self, unit_name, node, message):
        self.unit_name = unit_name
        self.astnode = node
        self.message = message

    def __repr__(self):
        return "Error in %r at line %r: %s" % (self.unit_name, self.astnode.lineno, self.message)

    def __str__(self):
        return repr(self)

class InspectError(NodeProcessingError):

    "An error during the module inspection process."

    pass

class TranslateError(NodeProcessingError):

    "An error during the module translation process."

    pass

class TranslationNotImplementedError(TranslateError):

    "An error caused by a node not being supported in translation."

    pass

# Useful data.

comparison_methods = {
    "==" : ("__eq__", "__eq__"),
    "!=" : ("__ne__", "__ne__"),
    "<" : ("__lt__", "__gt__"),
    "<=" : ("__le__", "__ge__"),
    ">=" : ("__ge__", "__le__"),
    ">" : ("__gt__", "__lt__"),
    "is" : None,
    "is not" : None,
    "in" : None,
    "not in" : None
    }

augassign_methods = {
    "+=" : ("__iadd__", ("__add__", "__radd__")),
    "-=" : ("__isub__", ("__sub__", "__rsub__")),
    "*=" : ("__imul__", ("__mul__", "__rmul__")),
    "/=" : ("__idiv__", ("__div__", "__rdiv__")),
    "//=" : ("__ifloordiv__", ("__floordiv__", "__rfloordiv__")),
    "%=" : ("__imod__", ("__mod__", "__rmod__")),
    "**=" : ("__ipow__", ("__pow__", "__rpow__")),
    "<<=" : ("__ilshift__", ("__lshift__", "__rlshift__")),
    ">>=" : ("__irshift__", ("__rshift__", "__rrshift__")),
    "&=" : ("__iand__", ("__and__", "__rand__")),
    "^=" : ("__ixor__", ("__xor__", "__rxor__")),
    "|=" : ("__ior__", ("__or__", "__ror__"))
    }

binary_methods = {
    "Add" : ("__add__", "__radd__"),
    "Bitand" : ("__and__", "__rand__"),
    "Bitor" : ("__or__", "__ror__"),
    "Bitxor" : ("__xor__", "__rxor__"),
    "Div" : ("__div__", "__rdiv__"),
    "FloorDiv" : ("__floordiv__", "__rfloordiv__"),
    "LeftShift" : ("__lshift__", "__rlshift__"),
    "Mod" : ("__mod__", "__rmod__"),
    "Mul" : ("__mul__", "__rmul__"),
    "Power" : ("__pow__", "__rpow__"),
    "RightShift" : ("__rshift__", "__rrshift__"),
    "Sub" : ("__sub__", "__rsub__")
    }

unary_methods = {
    "Invert" : "__invert__",
    "UnaryAdd" : "__pos__",
    "UnarySub" : "__neg__"
    }

# vim: tabstop=4 expandtab shiftwidth=4
